@@ -3,25 +3,27 @@
  *
  * File begun on 2007-07-30 by RGerhards
  *
- * Copyright (C) 2007-2012 Adiscon GmbH.
+ * Copyright (C) 2007, 2008 by Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
  * This file is part of the rsyslog runtime library.
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *       http://www.apache.org/licenses/LICENSE-2.0
- *       -or-
- *       see COPYING.ASL20 in the source distribution
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * The rsyslog runtime library is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * The rsyslog runtime library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with the rsyslog runtime library.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * A copy of the GPL can be found in the file "COPYING" in this distribution.
+ * A copy of the LGPL can be found in the file "COPYING.LESSER" in this distribution.
  */
 #include "config.h"
 
@@ -37,6 +39,7 @@
 
 #include "cfsysline.h"
 #include "obj.h"
+#include "conf.h"
 #include "errmsg.h"
 #include "srUtils.h"
 #include "unicode-helper.h"
@@ -656,11 +659,13 @@ static int cslchKeyCompare(void *pKey1, void *pKey2)
 
 /* set data members for this object
  */
-rsRetVal cslchSetEntry(cslCmdHdlr_t *pThis, ecslCmdHdrlType eType, rsRetVal (*pHdlr)(), void *pData)
+rsRetVal cslchSetEntry(cslCmdHdlr_t *pThis, ecslCmdHdrlType eType, rsRetVal (*pHdlr)(),
+	void *pData, ecslConfObjType eConfObjType)
 {
 	assert(pThis != NULL);
 	assert(eType != eCmdHdlrInvalid);
 
+	pThis->eConfObjType = eConfObjType;
 	pThis->eType = eType;
 	pThis->cslCmdHdlr = pHdlr;
 	pThis->pData = pData;
@@ -777,7 +782,8 @@ finalize_it:
 
 /* add a handler entry to a known command
  */
-static rsRetVal cslcAddHdlr(cslCmd_t *pThis, ecslCmdHdrlType eType, rsRetVal (*pHdlr)(), void *pData, void *pOwnerCookie)
+static rsRetVal cslcAddHdlr(cslCmd_t *pThis, ecslCmdHdrlType eType, rsRetVal (*pHdlr)(), void *pData,
+	void *pOwnerCookie, ecslConfObjType eConfObjType)
 {
 	DEFiRet;
 	cslCmdHdlr_t *pCmdHdlr = NULL;
@@ -785,7 +791,7 @@ static rsRetVal cslcAddHdlr(cslCmd_t *pThis, ecslCmdHdrlType eType, rsRetVal (*p
 	assert(pThis != NULL);
 
 	CHKiRet(cslchConstruct(&pCmdHdlr));
-	CHKiRet(cslchSetEntry(pCmdHdlr, eType, pHdlr, pData));
+	CHKiRet(cslchSetEntry(pCmdHdlr, eType, pHdlr, pData, eConfObjType));
 	CHKiRet(llAppend(&pThis->llCmdHdlrs, pOwnerCookie, pCmdHdlr));
 
 finalize_it:
@@ -805,7 +811,7 @@ finalize_it:
  * free pCmdName if he allocated it dynamically! -- rgerhards, 2007-08-09
  */
 rsRetVal regCfSysLineHdlr(uchar *pCmdName, int bChainingPermitted, ecslCmdHdrlType eType, rsRetVal (*pHdlr)(), void *pData,
-			  void *pOwnerCookie)
+			  void *pOwnerCookie, ecslConfObjType eConfObjType)
 {
 	DEFiRet;
 	cslCmd_t *pThis;
@@ -815,7 +821,7 @@ rsRetVal regCfSysLineHdlr(uchar *pCmdName, int bChainingPermitted, ecslCmdHdrlTy
 	if(iRet == RS_RET_NOT_FOUND) {
 		/* new command */
 		CHKiRet(cslcConstruct(&pThis, bChainingPermitted));
-		CHKiRet_Hdlr(cslcAddHdlr(pThis, eType, pHdlr, pData, pOwnerCookie)) {
+		CHKiRet_Hdlr(cslcAddHdlr(pThis, eType, pHdlr, pData, pOwnerCookie, eConfObjType)) {
 			cslcDestruct(pThis);
 			FINALIZE;
 		}
@@ -835,7 +841,7 @@ rsRetVal regCfSysLineHdlr(uchar *pCmdName, int bChainingPermitted, ecslCmdHdrlTy
 		if(pThis->bChainingPermitted == 0 || bChainingPermitted == 0) {
 			ABORT_FINALIZE(RS_RET_CHAIN_NOT_PERMITTED);
 		}
-		CHKiRet_Hdlr(cslcAddHdlr(pThis, eType, pHdlr, pData, pOwnerCookie)) {
+		CHKiRet_Hdlr(cslcAddHdlr(pThis, eType, pHdlr, pData, pOwnerCookie, eConfObjType)) {
 			cslcDestruct(pThis);
 			FINALIZE;
 		}
@@ -910,6 +916,7 @@ rsRetVal processCfSysLineCommand(uchar *pCmdName, uchar **p)
 	uchar *pHdlrP; /* the handler's private p (else we could only call one handler) */
 	int bWasOnceOK; /* was the result of an handler at least once RS_RET_OK? */
 	uchar *pOKp = NULL; /* returned conf line pointer when it was OK */
+	int bHadScopingErr = 0; /* set if a scoping error occured */
 
 	iRet = llFind(&llCmdList, (void *) pCmdName, (void*) &pCmd);
 
@@ -923,17 +930,25 @@ rsRetVal processCfSysLineCommand(uchar *pCmdName, uchar **p)
 	llCookieCmdHdlr = NULL;
 	bWasOnceOK = 0;
 	while((iRetLL = llGetNextElt(&pCmd->llCmdHdlrs, &llCookieCmdHdlr, (void*)&pCmdHdlr)) == RS_RET_OK) {
-		/* for the time being, we ignore errors during handlers. The
-		 * reason is that handlers are independent. An error in one
-		 * handler does not necessarily mean that another one will
-		 * fail, too. Later, we might add a config variable to control
-		 * this behaviour (but I am not sure if that is rally
-		 * necessary). -- rgerhards, 2007-07-31
-		 */
-		pHdlrP = *p;
-		if((iRet = cslchCallHdlr(pCmdHdlr, &pHdlrP)) == RS_RET_OK) {
-			bWasOnceOK = 1;
-			pOKp = pHdlrP;
+		/* check if handler is valid in current scope */
+		if(pCmdHdlr->eConfObjType == eConfObjAlways ||
+		   (bConfStrictScoping == 0 && currConfObj == eConfObjGlobal) ||
+		   pCmdHdlr->eConfObjType == currConfObj) {
+			/* for the time being, we ignore errors during handlers. The
+			 * reason is that handlers are independent. An error in one
+			 * handler does not necessarily mean that another one will
+			 * fail, too. Later, we might add a config variable to control
+			 * this behaviour (but I am not sure if that is really
+			 * necessary). -- rgerhards, 2007-07-31
+			 */
+			pHdlrP = *p;
+			if((iRet = cslchCallHdlr(pCmdHdlr, &pHdlrP)) == RS_RET_OK) {
+				bWasOnceOK = 1;
+				pOKp = pHdlrP;
+			}
+		} else {
+			errmsg.LogError(0, RS_RET_CONF_INVLD_SCOPE, "config command invalid for current scope");
+			bHadScopingErr = 1;
 		}
 	}
 
@@ -945,6 +960,10 @@ rsRetVal processCfSysLineCommand(uchar *pCmdName, uchar **p)
 	if(iRetLL != RS_RET_END_OF_LINKEDLIST)
 		iRet = iRetLL;
 
+	if(bHadScopingErr) {
+		iRet = RS_RET_CONF_INVLD_SCOPE;
+	}
+
 finalize_it:
 	RETiRet;
 }
